__author__ = 'Dmitriy Korsakov'

import os
import json
import pydoc

import yaml

from scalrctl import click
from scalrctl import commands
from scalrctl import settings


class Import(commands.Action):

    action = None
    epilog = "Example: scalr-ctl import < object.yml"

    def _init(self):
        super(Import, self)._init()

        with open(os.path.join(os.path.dirname(__file__), '../scheme/scheme.json')) as fp:
            self.scheme = json.load(fp)

    def get_description(self):
        return "Import Scalr Objects."

    def get_options(self):
        debug = click.Option(('--debug/--no-debug', 'debug'), default=False, help="Print debug messages")
        envid = click.Option(('--envId', 'env_id'), help="Environment ID")
        upd_helpmsg = "Update existing object instead of creating new."
        update = click.Option(('--update', 'update'), is_flag=True, default=False, help=upd_helpmsg)
        return [debug, update, envid]

    def run(self, *args, **kwargs):
        if "debug" in kwargs:
            settings.debug_mode = kwargs.pop("debug")
        env_id = kwargs.pop("env_id", None)

        raw = click.get_text_stream("stdin")
        obj_data = self._validate_object(raw)

        get_type_route = obj_data["meta"]["scalrctl"]["ROUTE"]
        http_method = "patch" if kwargs.pop("update", False) else "post"

        action_scheme = None
        for obj_name, section in self.scheme["export"].items():
            if obj_name != "group_descr" and section["http-method"] == "get" and section["route"] == get_type_route:
                action_scheme = section["%s-params" % http_method]

        if not action_scheme:
            raise click.ClickException("Cannot import Scalr object: API method (%s,%s) not found" % (
                action_scheme["http-method"],
                action_scheme["route"]))

        cls = pydoc.locate(action_scheme['class']) if 'class' in action_scheme else commands.Action
        action = cls(name=self.name, route=action_scheme['route'], http_method=http_method, api_level=self.api_level)
        arguments, kv = obj_data["meta"]["scalrctl"]['ARGUMENTS']
        kv["import-data"] = {action.get_body_type_params()[0]["name"]: obj_data["data"]}
        if env_id:
            kv["envId"] = env_id
        return action.run(*arguments, **kv)

    def _validate_object(self, yml):
        try:
            descr = yaml.load(yml)
            assert "data" in descr
            assert "meta" in descr
            meta = descr["meta"]
            assert "scalrctl" in meta
            info = meta["scalrctl"]

            for key in (
                "METHOD",
                "ROUTE",
                "envId",
                "ARGUMENTS",
                "API_LEVEL"
            ):
                assert key in info

            return descr

        except (Exception, BaseException) as e:

            if settings.debug_mode:
                click.echo(e)
                click.echo(descr)

            raise click.ClickException("Invalid object description")



__author__ = 'Dmitriy Korsakov'

import json
import datetime
import copy
from collections import OrderedDict

from scalrctl import settings
from scalrctl import commands
from scalrctl import click
from scalrctl import defaults

import yaml


class Export(commands.Action):

    def run(self, *args, **kwargs):
        kv = kwargs.copy()
        kv['hide_output'] = True
        response = super(Export, self).run(*args, **kv)

        kwargs["envId"] = settings.envId
        kv = copy.deepcopy(kwargs)
        for item in ("debug", "nocolor", "transformation"):
            if item in kv:
                del kv[item]
        uri = self._request_template.format(**kwargs)

        d = {
            "API_VERSION": settings.API_VERSION,
            "envId": settings.envId,
            "DATE": datetime.datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S"),
            "API_HOST": settings.API_HOST,
            "API_LEVEL": self.api_level,
            "METHOD": self.http_method,
            "ROUTE": self.route,
            "URI": uri,
            "ACTION": self.name,
            "ARGUMENTS": (args, kv),
            "SCALRCTL_VERSION": defaults.VERSION,
             }

        try:
            response_json = json.loads(response)
        except ValueError as e:
            if settings.debug_mode:
                raise
            raise click.ClickException(str(e))

        response_json["meta"]["scalrctl"] = d

        dump = yaml.safe_dump(
            response_json,
            encoding='utf-8',
            allow_unicode=True,
            default_flow_style=False
        )
        click.echo(dump)


class ExportFarmRoleGlobalVariable(Export):
    prompt_for = ["roleId", "globalVariableName"]


class ExportImage(Export):
    prompt_for = ["imageId"]


import textwrap
from contextlib import contextmanager


class TextWrapper(textwrap.TextWrapper):

    def _handle_long_word(self, reversed_chunks, cur_line, cur_len, width):
        space_left = max(width - cur_len, 1)

        if self.break_long_words:
            last = reversed_chunks[-1]
            cut = last[:space_left]
            res = last[space_left:]
            cur_line.append(cut)
            reversed_chunks[-1] = res
        elif not cur_line:
            cur_line.append(reversed_chunks.pop())

    @contextmanager
    def extra_indent(self, indent):
        old_initial_indent = self.initial_indent
        old_subsequent_indent = self.subsequent_indent
        self.initial_indent += indent
        self.subsequent_indent += indent
        try:
            yield
        finally:
            self.initial_indent = old_initial_indent
            self.subsequent_indent = old_subsequent_indent

    def indent_only(self, text):
        rv = []
        for idx, line in enumerate(text.splitlines()):
            indent = self.initial_indent
            if idx > 0:
                indent = self.subsequent_indent
            rv.append(indent + line)
        return '\n'.join(rv)



__author__ = 'Dmitriy Korsakov'
__doc__ = 'Server management'
import copy
from scalrctl import commands
from scalrctl import click


class RebootServer(commands.Action):

    epilog = "Example: scalr-ctl server reboot --serverId <ID> --hard"

    post_template = {
        "serverRebootOptions": {"hard": True}
    }

    def get_options(self):
        hlp = "Reboot type. By default it does soft reboot unless this \
        option is set to true. Beware that some types of the instances do \
        not support soft reboot."
        hard_reboot = click.Option(('--hard', 'hard'), is_flag=True, default=False, help=hlp)
        options = super(RebootServer, self).get_options()
        options.append(hard_reboot)
        return options


    def pre(self, *args, **kwargs):
        """
        before request is made
        """
        hard = kwargs.pop("hard", None)
        post_data = copy.deepcopy(self.post_template)
        post_data["serverRebootOptions"]["hard"] = hard
        kv = {"import-data": post_data}
        kv.update(kwargs)
        arguments, kw = super(RebootServer, self).pre(*args, **kv)
        return arguments, kw


class ResumeServer(commands.Action):

    epilog = "Example: scalr-ctl server resume --serverId <ID>"
    post_template = {}

    def pre(self, *args, **kwargs):
        """
        before request is made
        """
        kv = {"import-data": {}}
        kv.update(kwargs)
        arguments, kw = super(ResumeServer, self).pre(*args, **kv)
        return arguments, kw


class SuspendServer(commands.Action):

    epilog = "Example: scalr-ctl server suspend  --serverId <ID>"
    post_template = {}

    def pre(self, *args, **kwargs):
        """
        before request is made
        """
        kv = {"import-data": {}}
        kv.update(kwargs)
        arguments, kw = super(SuspendServer, self).pre(*args, **kv)
        return arguments, kw


class TerminateServer(commands.Action):

    epilog = "Example: scalr-ctl server terminate --serverId <ID> --force"
    post_template = {
        "serverTerminationOptions": {"hard": True}
    }

    def get_options(self):
        hlp = "It is used to terminate the Server immediately ignoring scalr.system.server_terminate_timeout."
        hard_terminate = click.Option(('--force', 'hard'), is_flag=True, default=False, help=hlp)
        options = super(TerminateServer, self).get_options()
        options.append(hard_terminate)
        return options


    def pre(self, *args, **kwargs):
        """
        before request is made
        """
        hard = kwargs.pop("hard", None)
        post_data = copy.deepcopy(self.post_template)
        post_data["serverTerminationOptions"]["hard"] = hard
        kv = {"import-data": post_data}
        kv.update(kwargs)
        arguments, kw = super(TerminateServer, self).pre(*args, **kv)
        return arguments, kw
